#include <stdio.h>

#include "config.h"
#include "imapfilter.h"
#include "account.h"
#include "filter.h"


extern account_t *accounts;
extern mboxgrp_t *mboxgrps;
extern filter_t *filters;

extern account_t *cur_acct;
extern filter_t *cur_fltr;


void destroy_mboxgrps(mboxgrp_t * node);
void destroy_mboxs(mbox_t * node);
void destroy_accounts(account_t * node);
void destroy_filters(filter_t * node);
void destroy_masks(mask_t * node);


/*
 * Free allocated memory of all data structures.
 */
void
destroy_all(void)
{
	destroy_accounts(accounts);
	accounts = NULL;
	cur_acct = NULL;
	destroy_filters(filters);
	filters = NULL;
	cur_fltr = NULL;
	destroy_mboxgrps(mboxgrps);
	mboxgrps = NULL;
}


/*
 * Free allocated memory of data structures that are not needed anymore.
 */
void
destroy_unneeded(void)
{
	destroy_mboxgrps(mboxgrps);
	mboxgrps = NULL;
}


/*
 * Go through the mailbox-group tree, and free the allocated memory of
 * each node.
 */
void
destroy_mboxgrps(mboxgrp_t * node)
{
	if (node == NULL)
		return;

	if (node->left != NULL) {
		destroy_mboxgrps(node->left);
		node->left = NULL;
	}
	if (node->right != NULL) {
		destroy_mboxgrps(node->right);
		node->right = NULL;
	}
#ifdef DEBUG
	fprintf(stderr, "debug: deleting FOLDER: '%s'\n", node->key);
#endif

	xfree(node);
}


/*
 * Go through the mailbox linked list of the account and free the allocated
 * memory of each node.
 */
void
destroy_mboxs(mbox_t * node)
{
	mbox_t *p, *t;

	for (p = node; p != NULL; p = t) {
		t = p->next;
#ifdef DEBUG
		fprintf(stderr, "debug: deleting MBOX: '%s'\n", p->name);
#endif
		xfree(p);
	}
}


/*
 * Go through the accounts' linked list and free the allocated memory of
 * each node.
 */
void
destroy_accounts(account_t * node)
{
	account_t *p, *t;

	for (p = node; p != NULL; p = t) {
		t = p->next;
#ifdef DEBUG
		fprintf(stderr, "debug: deleting ACCOUNT: '%s'\n", p->key);
#endif
		destroy_mboxs(p->mboxes);
		sfree(p->password);
		xfree(p);
	}
}


/*
 * Go through the filters' tree and free the allocated memory of each node.
 */
void
destroy_filters(filter_t * node)
{
	if (node == NULL)
		return;

	if (node->left != NULL) {
		destroy_filters(node->left);
		node->left = NULL;
	}
	if (node->right != NULL) {
		destroy_filters(node->right);
		node->right = NULL;
	}
#ifdef DEBUG
	fprintf(stderr, "debug: deleting FILTER: '%s'\n", node->key);
#endif
	destroy_masks(node->masks);
	xfree(node);
}


/*
 * Go through the masks' linked list and free the allocated memory of each
 * node.
 */
void
destroy_masks(mask_t * node)
{
	mask_t *p, *t;

	for (p = node; p != NULL; p = t) {
		t = p->next;
#ifdef DEBUG
		fprintf(stderr, "debug: deleting MASK: '%s'\n", p->body);
#endif
		xfree(p);
	}
}
